/* comb reverb parameters */
    /* number of combs */
    /* JAM's values for Rz at 50KHz sampling rate - terminate with -1.0 */
#define CMBDLYS {.05, .056, .061, .068, .072, .078, -1.0}
    /* JAM's values for g1 at 50KHz sampling rate, g2 = 1 - g1 for g1 > 0 */
#define CMBG (0.9)
#define CMBG1 {.46, .48, .50, .52, .53, .55} 
