/* allpass reverb parameters */
#define APDLYS {.006, -1.0}
#define APG1 {.7}
/*define APDLYS {.006, 0.0091, .0099, -1.0}*/
/*define APG1 {.7, .683, .721}*/
/* the allpass G2 term is calculated automatically */
